@@ -3,6 +3,7 @@ import sys
 import json
 import math
 import torch
+import random
 import hashlib
 import inspect
 import argparse
@@ -66,6 +67,28 @@ except:
 # general
 
 
+min_seed_value = np.iinfo(np.uint32).min
+max_seed_value = np.iinfo(np.uint32).max
+
+
+def new_seed() -> int:
+    return random.randint(min_seed_value, max_seed_value)
+
+
+def seed_everything(seed: int) -> int:
+    if not min_seed_value <= seed <= max_seed_value:
+        msg = f"{seed} is not in bounds, numpy accepts from {min_seed_value} to {max_seed_value}"
+        print_warning(msg)
+        seed = new_seed()
+
+    random.seed(seed)
+    np.random.seed(seed)
+    torch.manual_seed(seed)
+    torch.cuda.manual_seed_all(seed)
+
+    return seed
+
+
 def _get_environ_workplace() -> Optional[str]:
     return os.environ.get(WORKPLACE_ENVIRON_KEY)
 
@@ -1018,16 +1041,27 @@ def slerp(
     x2: torch.Tensor,
     r1: Union[float, torch.Tensor],
     r2: Optional[Union[float, torch.Tensor]] = None,
+    *,
+    dot_threshold: float = 0.9995,
 ) -> torch.Tensor:
-    low_norm = x1 / torch.norm(x1, dim=1, keepdim=True)
-    high_norm = x2 / torch.norm(x2, dim=1, keepdim=True)
-    omega = torch.acos((low_norm * high_norm).sum(1))
-    so = torch.sin(omega)
     if r2 is None:
         r2 = 1.0 - r1
+    b, *shape = x1.shape
+    x1 = x1.view(b, -1)
+    x2 = x2.view(b, -1)
+    low_norm = x1 / torch.norm(x1, dim=1, keepdim=True)
+    high_norm = x2 / torch.norm(x2, dim=1, keepdim=True)
+    dot = (low_norm * high_norm).sum(1)
+    overflow_mask = dot > dot_threshold
+    out = torch.zeros_like(x1)
+    out[overflow_mask] = r1 * x1 + r2 * x2
+    normal_mask = ~overflow_mask
+    omega = torch.acos(dot[normal_mask])
+    so = torch.sin(omega)
     x1_part = (torch.sin(r1 * omega) / so).unsqueeze(1) * x1
     x2_part = (torch.sin(r2 * omega) / so).unsqueeze(1) * x2
-    return x1_part + x2_part
+    out[normal_mask] = x1_part + x2_part
+    return out.view(b, *shape)
 
 
 def interpolate(
