@@ -7,7 +7,7 @@ from typing import List
 from typing import Union
 from typing import Optional
 
-from ..protocol import ImageTranslatorMixin
+from ...protocols.cv import ImageTranslatorMixin
 from ....misc.toolkit import interpolate
 from ....modules.blocks import get_clones
 from ....modules.blocks import get_conv_blocks
@@ -143,8 +143,8 @@ class UNetFRS(UNetBase):
         return self.msg
 
 
-@register_module("u2net", pre_bases=[ImageTranslatorMixin])
-class U2Net(UNetBase):
+@register_module("u2net")
+class U2Net(UNetBase, ImageTranslatorMixin):
     def __init__(
         self,
         in_channels: int,
